/*
* ============================================================================
* History
* =======
*
* (C) Copyright 2014, Future Technology Devices International Ltd.
* ============================================================================
*
* This source code ("the Software") is provided by Future Technology Devices
* International Limited ("FTDI") subject to the licence terms set out
* http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
* You must read the Licence Terms before downloading or using the Software.
* By installing or using the Software you agree to the Licence Terms. If you
* do not agree to the Licence Terms then do not download or use the Software.
*
* Without prejudice to the Licence Terms, here is a summary of some of the key
* terms of the Licence Terms (and in the event of any conflict between this
* summary and the Licence Terms then the text of the Licence Terms will
* prevail).
*
* The Software is provided "as is".
* There are no warranties (or similar) in relation to the quality of the
* Software. You use it at your own risk.
* The Software should not be used in, or for, any medical device, system or
* appliance. There are exclusions of FTDI liability for certain types of loss
* such as: special loss or damage; incidental loss or damage; indirect or
* consequential loss or damage; loss of income; loss of business; loss of
* profits; loss of revenue; loss of contracts; business interruption; loss of
* the use of money or anticipated savings; loss of information; loss of
* opportunity; loss of goodwill or reputation; and/or loss of, damage to or
* corruption of data.
* There is a monetary cap on FTDI's liability.
* The Software may have subsequently been amended by another user and then
* distributed by that other user ("Adapted Software").  If so that user may
* have additional licence terms that apply to those amendments. However, FTDI
* has no liability in relation to those amendments.
* ============================================================================
*/

/* This file contains SPI HAL Apis */

#include "FT_DataTypes.h"
#include "FT_Gpu.h"
#include "FT_Hal_Spi.h"

/* Libmpsse specific inclusions */
#include <stdio.h>
#include <stdlib.h>
/* Include D2XX header*/
#include "ftd2xx.h"
/* Include libMPSSE header */
#include "libMPSSE_spi.h"
#include "FT_Hal_Spi.h"

#define ARDUINO_PRO_SPI_CS   9
#define FT_CMD_FIFO_SIZE     4096
#define FT_JPEG_BLOCK_SIZE    2048


#define CURRENT_FT_CMDFIFO_WP (RAM_CMD + __ft_cmd_fifo_wp)
#define CURRENT_FT_DLBUFF_WP  (RAM_DL + __ft_dl_buff_wp)

ft_uint16_t __ft_cmd_fifo_wp = 0; //vc coprocessor fifo write pointer
ft_uint16_t __ft_dl_buff_wp = 0;  //vc display command memory write pointer

ft_int16_t __get_cmdfifo_freespace();
ft_void hal_spi_wrcmd32s(FT_PROGMEM prog_uchar *src, ft_uint32_t count);

uint32 channels;					//number of channels connected 
FT_HANDLE ftHandle;					//handle used by d2xx

/******************************************************************************/
/* SPI master apis to directly access SPI IP                                  */
/* Directly configure spi master registers  */
/******************************************************************************/

/* API to reset the write pointer maintained by the MCU */
ft_void hal_spi_resetwp()
{
    __ft_cmd_fifo_wp = 0;
}

/* API to transfer one byte of data through SPI */
ft_uint8_t SPI_Transfer_Byte(ft_uint8_t wrbyte)
{

	  /* Transfer one byte via MPSSE SPI interface as well as receive one byte */
	  ft_uint8_t Transfer_Array[4],Receive_Array[4];
	  ft_uint32_t SizeTransfered;
	  Transfer_Array[0] = wrbyte;
	  SPI_ReadWrite(ftHandle,Receive_Array,Transfer_Array,1,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES);

	  return Receive_Array[0];
}

/* API to initialize the SPI interface */
ft_void hal_spi_init()
{
    ChannelConfig channelConf;			//channel configuration
	FT_STATUS status;					//return status of apis
	ft_int32_t i;
	FT_DEVICE_LIST_INFO_NODE devList;

	/* Initialize the libmpsse */
    Init_libMPSSE();		
	/* configure the spi settings */
	channelConf.ClockRate = 12000000;
	channelConf.LatencyTimer= 2;
	channelConf.configOptions = SPI_CONFIG_OPTION_MODE0 | SPI_CONFIG_OPTION_CS_DBUS3 | SPI_CONFIG_OPTION_CS_ACTIVELOW;
	channelConf.Pin = 0x00000000;	/*FinalVal-FinalDir-InitVal-InitDir (for dir 0=in, 1=out)*/

	status = SPI_GetNumChannels(&channels);
	printf("Number of available SPI channels = %d\n",channels);
	/* By default i am assuming only one mpsse cable is connected to PC and channel 0 of that mpsse cable is used for spi transactions */
	if(channels > 0)
	{
		status = SPI_GetChannelInfo(0,&devList);
		printf("Information on channel number %d:\n",0);
		/* print the dev info */
		printf(" Flags=0x%x\n",devList.Flags);
		printf(" Type=0x%x\n",devList.Type);
		printf(" ID=0x%x\n",devList.ID);
		printf(" LocId=0x%x\n",devList.LocId);
		printf(" SerialNumber=%s\n",devList.SerialNumber);
		printf(" Description=%s\n",devList.Description);
		printf(" ftHandle=0x%x\n",devList.ftHandle);/*is 0 unless open*/
	}
	/* Open the first available channel */
	status = SPI_OpenChannel(0,&ftHandle);
	printf("\nhandle=0x%x status=0x%x\n",ftHandle,status);
	status = SPI_InitChannel(ftHandle,&channelConf);
	printf("\nhandle=0x%x status=0x%x\n",ftHandle,status);

	/* Set the command fifo buffer offset to 0 */
  __ft_cmd_fifo_wp = __ft_dl_buff_wp = 0;
}

/* Close spi handle */
ft_void hal_spi_exit()
{
	FT_STATUS status; 
	/* Clean off the lib mpsse */
	status = SPI_CloseChannel(ftHandle);
	Cleanup_libMPSSE();
}


/* API for SD card CS enable 
ft_void SPI_CS_SD_Assert()
{
}

 SPI for SD card CS diable 
ft_void SPI_CS_SD_Dessert()
{


}
*/


/* Program to use external clock as well as set the clock to 48MHz */
ft_void hal_spi_clkconfig()
{
	  ft_uint8_t Transfer_Array[2];
	  ft_uint32_t SizeTransfered;
	  // sleep(1);
	  
	  /* Set the clk to external clock */
	  Transfer_Array[0] = 0x44;
	  Transfer_Array[1] = 0;
	  SPI_Write(ftHandle,Transfer_Array,2,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);
	  // sleep(1);

	  /* change of clock command */
	  Transfer_Array[0] = 0x62;
	  Transfer_Array[1] = 0;
	  SPI_Write(ftHandle,Transfer_Array,2,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);

	// sleep(1);
}
/* Do a core reset */
ft_void hal_spi_corerst()
{
	{
	  ft_uint8_t Transfer_Array[2];
	  ft_uint32_t SizeTransfered;
	// sleep(1);
	  
	  /* change of clock command */
	  Transfer_Array[0] = 0x68;
	  Transfer_Array[1] = 0;
	  SPI_Write(ftHandle,Transfer_Array,2,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);
	// sleep(1);

	}

}
ft_void hal_spi_rdstart(ft_uint32_t addr)
{
  ft_uint8_t Transfer_Array[3];
  ft_uint32_t SizeTransfered;

  //__rdwr_start_addr = addr;
  /* Compose the read packet */
  Transfer_Array[0] = addr >> 16;
  Transfer_Array[1] = addr >> 8;
  Transfer_Array[2] = addr;
  SPI_Write(ftHandle,Transfer_Array,3,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE);
/* Store the read pointer */
}

ft_void hal_spi_wrstart(ft_uint32_t addr)
{
  //__rdwr_start_addr = addr;
  ft_uint8_t Transfer_Array[3];
  ft_uint32_t SizeTransfered;

  //__rdwr_start_addr = addr;
  /* Compose the read packet */
  Transfer_Array[0] = (0x80 | (addr >> 16));
  Transfer_Array[1] = addr >> 8;
  Transfer_Array[2] = addr;
  SPI_Write(ftHandle,Transfer_Array,3,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE);

}

/* Coprocessor FIFO specific write api */
ft_void  hal_spi_wrcmdstart(ft_int16_t count)
{
        ft_int16_t getfreespace;

        do{
                getfreespace = __get_cmdfifo_freespace();
    }while(getfreespace < count);
  hal_spi_wrstart(CURRENT_FT_CMDFIFO_WP);
}


ft_void hal_spi_end()
{
  /* add code to change only the CS from enable to disable */
  ft_uint8_t Transfer_Array[4];
  ft_uint32_t SizeTransfered;

  //just disbale the CS - send 0 bytes with CS disable
  SPI_Write(ftHandle,Transfer_Array,1,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BITS | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);

}

ft_uint8_t hal_spi_rd8(ft_uint32_t addr)
{
  ft_uint8_t Receive_Array[4];
  ft_uint32_t SizeTransfered;

  ft_uint8_t ret;
  hal_spi_rdstart(addr);
  Receive_Array[0] = 0;
  SPI_Read(ftHandle,Receive_Array,2,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);

  //hal_spi_end();//optimization
  ret = Receive_Array[1];

  return ret;
}
ft_uint16_t hal_spi_rd16(ft_uint32_t addr)
{
  ft_uint16_t ret = 0;
  ft_uint8_t Receive_Array[4];
  ft_uint32_t SizeTransfered;

  hal_spi_rdstart(addr);
  Receive_Array[0] = 0;
  SPI_Read(ftHandle,Receive_Array,3,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);

  //hal_spi_end();//optimization - combining cs and data transfer in SPI_Read api
  ret = Receive_Array[1];
  ret |= (Receive_Array[2] << 8);
  return ret;
}


ft_int16_t __get_cmdfifo_freespace()
{
  ft_int16_t fullness,retval;

        fullness = (__ft_cmd_fifo_wp - hal_spi_rd16(REG_CMD_READ)) & 4095;
        retval = (FT_CMD_FIFO_SIZE - 4) - fullness;
  return (retval);
}

ft_uint32_t hal_spi_rd32(ft_uint32_t addr)
{
  ft_uint32_t ret = 0;
  ft_uint8_t Receive_Array[8];
  ft_uint32_t SizeTransfered;

  hal_spi_rdstart(addr);
  Receive_Array[0] = 0;
  //one dummy byte followed by 4 bytes
  SPI_Read(ftHandle,Receive_Array,5,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);

  //hal_spi_end();//optimization

  ret = Receive_Array[1];
  ret |= Receive_Array[2] << 8;
  ret |= Receive_Array[3] << 16;
  ret |= Receive_Array[4] << 24;
  return ret;
}

/* Read data into read buffer - return number of bytes read */
ft_uint32_t hal_spi_rd8s(ft_uint32_t addr,ft_uint8_t *dst,ft_uint32_t count)
{
  ft_uint8_t Transfer_Array[4];
  ft_uint32_t SizeTransfered,SizeReceived;

  /* Compose the read packet */
  Transfer_Array[0] = addr >> 16;
  Transfer_Array[1] = addr >> 8;
  Transfer_Array[2] = addr;
  Transfer_Array[3] = 0;//dummy byte
  SPI_Write(ftHandle,Transfer_Array,4,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE);

  SPI_Read(ftHandle,dst,count,&SizeReceived,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);

  return SizeReceived;
  //TBD for other platforms

}
/* Write related APIs */


ft_void hal_spi_wr8(ft_uint32_t addr, ft_uint8_t v)
{
  ft_uint8_t Transfer_Array[5];
  ft_uint32_t SizeTransfered;

  Transfer_Array[0] = (0x80 | (addr >> 16));
  Transfer_Array[1] = addr >> 8;
  Transfer_Array[2] = addr;
  Transfer_Array[3] = v;
  SPI_Write(ftHandle,Transfer_Array,4,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);

}
ft_void hal_spi_wr16(ft_uint32_t addr, ft_uint16_t v)
{
  ft_uint8_t Transfer_Array[5];
  ft_uint32_t SizeTransfered;

  Transfer_Array[0] = (0x80 | (addr >> 16));
  Transfer_Array[1] = addr >> 8;
  Transfer_Array[2] = addr;
  Transfer_Array[3] = v;
  Transfer_Array[4] = v >> 8;
  SPI_Write(ftHandle,Transfer_Array,5,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);

}

ft_void hal_spi_wr32(ft_uint32_t addr, ft_uint32_t v)
{

  ft_uint8_t Transfer_Array[8];
  ft_uint32_t SizeTransfered;

  Transfer_Array[0] = (0x80 | (addr >> 16));
  Transfer_Array[1] = addr >> 8;
  Transfer_Array[2] = addr;
  Transfer_Array[3] = v;
  Transfer_Array[4] = v >> 8;
  Transfer_Array[5] = v >> 16;
  Transfer_Array[6] = v >> 24;

  SPI_Write(ftHandle,Transfer_Array,7,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);
}

/* API to transfer count number of bytes */
ft_void hal_spi_wr8s(ft_uint32_t addr,ft_uint8_t *src, ft_uint32_t count)
{
  ft_uint8_t Transfer_Array[3];
  ft_uint32_t SizeTransfered;

  Transfer_Array[0] = (0x80 | (addr >> 16));
  Transfer_Array[1] = addr >> 8;
  Transfer_Array[2] = addr;

  SPI_Write(ftHandle,Transfer_Array,3,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE);
  SPI_Write(ftHandle,src,count,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);
}

ft_int32_t hal_strlen(const ft_char8_t *s);
//write the length bytes in buffer to SPI
ft_void hal_spi_transfer_string_const(const ft_char8_t *s)
{
  ft_int16_t length = 0;
  ft_uint16_t i;
  ft_char8_t *prts;

  length = hal_strlen(s);
	/* transfer the string followed by null terminated character */
  ft_uint32_t SizeTransfered;
  ft_uint8_t Transfer_Array[4];
  SPI_Write(ftHandle,s,length,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES);
  Transfer_Array[0] = 0;
  SPI_Write(ftHandle,Transfer_Array,1,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES);
}


//write the length bytes in buffer to SPI
ft_void hal_spi_transfer_string(ft_char8_t *s)
{
  ft_uint32_t SizeTransfered,SizeToTransfer;
  ft_uint8_t Transfer_Array[5];

  SizeToTransfer = strlen(s);
  SPI_Write(ftHandle,s,SizeToTransfer,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES);
  Transfer_Array[0] = 0;
  SPI_Write(ftHandle,Transfer_Array,1,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES);

}

/* Transfer dummy byte */
ft_void hal_spi_insertdummy()
{
  ft_uint32_t SizeTransfered,SizeToTransfer;
  ft_uint8_t Transfer_Array[4];

  Transfer_Array[0] = 0;
  SPI_Write(ftHandle,Transfer_Array,1,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES);
}

/* Receive a byte from SPI */
ft_uint8_t hal_spi_receive8()
{
  ft_uint32_t SizeTransfered,SizeToTransfer;
  ft_uint8_t Receive_Array[4],ret;

  Receive_Array[0] = 0;
  SPI_Read(ftHandle,Receive_Array,1,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES);
  ret = Receive_Array[0];
  return ret;
 //     return 0;
}

ft_void hal_spi_transfer8(ft_uint8_t v)
{
  ft_uint32_t SizeTransfered,SizeToTransfer;
  ft_uint8_t Transfer_Array[5],ret;

  Transfer_Array[0] = v;
  SPI_Write(ftHandle,Transfer_Array,1,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES);

}
ft_void hal_spi_transfer16(ft_uint16_t v)
{

  ft_uint32_t SizeTransfered,SizeToTransfer;
  ft_uint8_t Transfer_Array[4],ret;

  Transfer_Array[0] = v;
  Transfer_Array[1] = v >> 8;
  SPI_Write(ftHandle,Transfer_Array,2,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES);

}

//write the length bytes in buffer to SPI
ft_void hal_spi_transfer32(ft_uint32_t v)
{
  ft_uint32_t SizeTransfered,SizeToTransfer;
  ft_uint8_t Transfer_Array[4],ret;

  Transfer_Array[0] = v;
  Transfer_Array[1] = v >> 8;
  Transfer_Array[2] = v >> 16;
  Transfer_Array[3] = v >> 24;
  SPI_Write(ftHandle,Transfer_Array,4,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES);

}


///////////////////////////////////////////////////////////////////////
/* APIs related to command fifo buffer implementation */

/* API to update the mcu globals wrt j1 fifo registers */
ft_void hal_spi_updatecmdfifo(ft_uint16_t count)
{
   __ft_cmd_fifo_wp  = (__ft_cmd_fifo_wp + count) & 4095;

   //4 byte alignment
   __ft_cmd_fifo_wp = (__ft_cmd_fifo_wp + 3) & 0xffc;
   hal_spi_wr16(REG_CMD_WRITE,__ft_cmd_fifo_wp);

   //calculate the free space of cmd fifo.
   //__ft_cmd_fifo_freespace -= (count & 0xffc);
   //__getfreespace();
}

/* API to write a single command into FIFO */
ft_void hal_spi_wrcmd32(ft_uint32_t c)
{
        ft_int16_t getfreespace;
        do{
                getfreespace = __get_cmdfifo_freespace();

    }while(getfreespace < sizeof(c));
   hal_spi_wr32(CURRENT_FT_CMDFIFO_WP,c);

   hal_spi_updatecmdfifo(sizeof(c));
}

/* Wait for the complete consumption of J1 commands */
ft_void hal_spi_waitcmdfifo_empty()
{
        ft_int16_t cmdrdptr,cmdwrptr;

        do{
                cmdrdptr = hal_spi_rd16(REG_CMD_READ);
                cmdwrptr = hal_spi_rd16(REG_CMD_WRITE);
    }while (cmdwrptr != cmdrdptr);
        __ft_cmd_fifo_wp = hal_spi_rd16(REG_CMD_WRITE);
}

/* Wait for the read write pointers to be same */
ft_void hal_spi_cmdidle()
{
        hal_spi_waitcmdfifo_empty();
}

ft_void hal_spi_wrcmd32s(FT_PROGMEM prog_uchar *src, ft_uint32_t count)
{
  ft_int32_t length;
  ft_uint16_t getfreespace;

  length = count;
  while (count > FT_JPEG_BLOCK_SIZE) {
    hal_spi_wrcmd32s(src, FT_JPEG_BLOCK_SIZE);

    src += FT_JPEG_BLOCK_SIZE;
    count -= FT_JPEG_BLOCK_SIZE;
  }

  length = count;
        do{
                getfreespace = __get_cmdfifo_freespace();
    }while(getfreespace < count);

  hal_spi_wrstart(CURRENT_FT_CMDFIFO_WP);

  ft_uint32_t SizeTransfered;
  SPI_Write(ftHandle,src,count,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES);

  hal_spi_end();//check whether this works with mpsse
  hal_spi_updatecmdfifo(length);
  hal_spi_cmdidle();
}

ft_void hal_spi_dump_cmdfifo(ft_uint8_t FT_PROGMEM*src, ft_uint16_t count)
{
    ft_uint16_t freespace = __get_cmdfifo_freespace();

    ft_uint16_t length = 0;

    while(count){

      while(!__get_cmdfifo_freespace());

      freespace = __get_cmdfifo_freespace();

      if (freespace > count)
         freespace = count;

      length = freespace;
/* For libmpsse try to call SPI_write api with complete buffer */
	ft_uint32_t SizeTransfered;
	ft_uint8_t Transfer_Array[4];

	Transfer_Array[0] = (0x80 | ((ft_uint32_t)(CURRENT_FT_CMDFIFO_WP) >> 16 ));
	Transfer_Array[1] = (ft_uint32_t)(CURRENT_FT_CMDFIFO_WP) >> 8; Transfer_Array[2] = (ft_uint32_t)(CURRENT_FT_CMDFIFO_WP);
	SPI_Write(ftHandle,Transfer_Array,3,&SizeTransfered,(SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE) );
	SPI_Write(ftHandle,src,freespace,&SizeTransfered,(SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE) );
	freespace -= SizeTransfered;
	hal_spi_updatecmdfifo(length);
	hal_spi_cmdidle();
      	count -= length;
  }
}
//  API to not to wait for the J1 to consume the complete data from FIFO 
ft_void hal_spi_wrcmd32s_nowaitidle(FT_PROGMEM prog_uchar *src, ft_uint32_t count)
{
  ft_int32_t length;
  ft_uint16_t getfreespace;

  length = count;
  while (count > FT_JPEG_BLOCK_SIZE) {
    hal_spi_wrcmd32s_nowaitidle(src, FT_JPEG_BLOCK_SIZE);

    src += FT_JPEG_BLOCK_SIZE;
    count -= FT_JPEG_BLOCK_SIZE;
  }

  length = count;
        do{
                getfreespace = __get_cmdfifo_freespace();
    }while(getfreespace < count);

  hal_spi_wrstart(CURRENT_FT_CMDFIFO_WP);

  ft_uint32_t SizeTransfered;
  SPI_Write(ftHandle,src,count,&SizeTransfered,SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES);


  hal_spi_end();
  hal_spi_updatecmdfifo(length);
}

ft_void hal_spi_download_jpeg(FT_PROGMEM prog_uchar *src, ft_uint16_t count)
{
  hal_spi_wrcmd32s(src,count);

}

/*  DL Api to make life easy and index error free */
ft_uint32_t hal_gpu_dli;
ft_void hal_gpu_dl(ft_uint32_t cmd)
{
	hal_spi_wr32(RAM_DL + hal_gpu_dli, cmd);
	hal_gpu_dli += 4;
}

ft_void hal_dl_start()
{
   hal_spi_wrcmd32(CMD_DLSTART);//reset the REG_CMD_DL to zero
   __ft_dl_buff_wp = 0;
}

ft_void hal_dl(ft_uint32_t c)
{
   hal_spi_cmdidle();

   __ft_dl_buff_wp += hal_spi_rd32(REG_CMD_DL);
   if (__ft_dl_buff_wp > 8195)
   {
     while(1);
   }

   hal_spi_wr32(CURRENT_FT_DLBUFF_WP,c);
   __ft_dl_buff_wp = (__ft_dl_buff_wp + 4) & 8195;//2048*4 -1
}

ft_void hal_dl_end()
{
   __ft_dl_buff_wp = 0;
   hal_spi_wr32(REG_DLSWAP,DLSWAP_FRAME);
}


/* implementation to get the string length for flash memory - pic and microe specific */
ft_int32_t hal_strlen(const ft_char8_t *s)
{
    ft_int32_t Charlen = 0;
    while(*s++ != 0)
    {
        Charlen++;
        if(Charlen > 128*1024)
        {
             /* error case */
             Charlen = 0;
             break;
        }
    }
    
    return Charlen;
}
/* implementation specific for microe - accessing memory of flash */
ft_char8_t* hal_strcpy(ft_char8_t *dst, const ft_char8_t *src)
{
    ft_int32_t Charlen = 0;
    ft_char8_t *p_dst;
    
    p_dst = dst;
    while(*src != 0)
    {
        *p_dst++ = *src++;
        Charlen++;
        if(Charlen > 128*1024)
        {
             /* error case */
             Charlen = 0;
             break;
        }
    }
    /* last ft_uint8_t as null terminating char */
    *p_dst = 0;
    return dst;
}
/* Comparison of num bytes in memory */
ft_char8_t hal_memcmp(ft_char8_t *p1,const ft_char8_t *p2,ft_int32_t num)
{
        ft_int32_t i;
        for(i=0;i<num;i++)
        {
                if(*p1++ != *p2++)
                {
                        return 1;
                }
        }
        /* no diff in memory */
        return 0;
}
ft_char8_t hal_memcmp_sram(ft_char8_t *p1,ft_char8_t *p2,ft_int32_t num)
{
        ft_int32_t i;
        for(i=0;i<num;i++)
        {
                if(*p1++ != *p2++)
                {
                        return 1;
                }
        }
        /* no diff in memory */
        return 0;
}
ft_char8_t hal_toupper(ft_char8_t inputchar)
{
        ft_char8_t outputchar;
        outputchar = inputchar;
        if((inputchar >= 'a') && (inputchar <= 'z'))
        {
                outputchar -= 0x32;
        }
        return outputchar;
}


/* Nothing beyond this */


